-- print("Modules/HistoryFrame.lua start")
--[[
    BASED ON:
	Auctioneer - Price Level Utility module
	Version: 5.12.5198 (QuirkyKiwi)
	Revision: $Id: CompactUI.lua 5016 2010-11-13 13:07:18Z brykrys $
	URL: http://auctioneeraddon.com/

	This is an addon for World of Warcraft that adds a price level indicator
	to auctions when browsing the Auction House, so that you may readily see
	which items are bargains or overpriced at a glance.

	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
		You have an implicit license to use this AddOn with these facilities
		since that is its designated purpose as per:
		http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]
local AceGUI = LibStub("AceGUI-3.0")

LFMonitor.historyMax = 100

function LFMonitor.classifierResultToText(result, type)
	local sensitivity = LFMonitorDB.options.sensitivity
	local known, text = false, ""
	local score, class
	for class, score in pairs(result[type]) do
		if score <= sensitivity then
			if known then
				text = text .. "/"
			end
			local classAbbr = LFMonitor.ABBRS[type][class] or class
			text = text .. classAbbr
			known = true
		end
	end
	if not known then
		return LFMonitor.TABLE_UNKNOWN
	else
		return text
	end
end

--[[ Local functions ]]--

LFMonitor.processedHistory = {}
LFMonitor.sortedHistory = {}
LFMonitor.historyTable = {}
LFMonitor.historyRows = {}
LFMonitor.sortKey = "Timestamp"
LFMonitor.sortDir = -1

LFMonitor.historyFilter = {
			["LFPROF"]  = true,
			["LFW"]     = true,
			["LFM"]     = true,
			["LFG"]     = true,
			["WTB"]     = true,
			["WTS"]     = true,
			["LFGUILD"] = true,
			["GUILDLFM"]= true,
			["unknown"]	= false
}

local function SetMessage(row, message, pos)
	local cell
	if message then
		local maxHeight = 0
		local k,v,height
		for k,v in pairs(message.display) do
			cell = LFMonitor.historyTable[pos][k]
			cell:SetText(v)
			cell:SetUserData("player",message.display.Player)
			height = cell.frame:GetHeight()
			if height > maxHeight then maxHeight = height end
		end
		row:SetHeight(maxHeight+6)
		row:GetUserData("texture"):SetTexture(pos%2,pos%2,pos%2, 0.1)
	else
		for _, cell in pairs(LFMonitor.historyTable[pos]) do
			cell:SetText("")
		end
		row:SetHeight(1)
		row:GetUserData("texture"):SetTexture(0,0,0,0)
	end
end

local function updateMessages()
	local i
	for i=1,LFMonitor.historyMax do
		SetMessage(LFMonitor.historyRows[i], LFMonitor.sortedHistory[i], i)
	end
end

local function comparer(v1, v2)
	if LFMonitor.sortDir > 0 then
		return (v1.sort[LFMonitor.sortKey] < v2.sort[LFMonitor.sortKey])
	else
		return (v2.sort[LFMonitor.sortKey] < v1.sort[LFMonitor.sortKey])
	end
end

local function resort()
	local sensitivity = LFMonitorDB.options.sensitivity
	local sortedHistory = {}
	local i,v,class,score
	for i,v in ipairs(LFMonitor.processedHistory) do
		local unknown = true
		for class, score in pairs(v.filter.Type) do
			if score <= sensitivity and LFMonitor.historyFilter[class] then
				unknown = false
				tinsert(sortedHistory, v)
				break
			end
		end
		if unknown and LFMonitor.historyFilter["unknown"] then
			tinsert(sortedHistory, v)
		end
	end
	table.sort(sortedHistory, comparer)
	LFMonitor.sortedHistory = sortedHistory
	if LFMonitor.HistoryFrame then
		updateMessages()
	end
end

local function reverseSort()
	local reversed = {}
	local m = #(LFMonitor.sortedHistory)
	local n
	for n=1,m do
		reversed[n] = LFMonitor.sortedHistory[m+1-n]
	end
	LFMonitor.sortedHistory = reversed
	if LFMonitor.HistoryFrame then
		updateMessages()
	end
end

function LFMonitor:processMessageForHistory(id,message,result)
	local output = {}
	output.display={
		Timestamp=LFMonitor:GetLocalTime(message.mtime),
		Channel=message.mchannel,
		Count=message.count,
		Player=message.msender,
		Type=LFMonitor.classifierResultToText(result,"type"),
		Profession=LFMonitor.classifierResultToText(result,"prof"),
		Class=LFMonitor.classifierResultToText(result,"class"),
		Role=LFMonitor.classifierResultToText(result,"role"),
		Raid=LFMonitor.classifierResultToText(result,"raid"),
		Message=message.mmsg
	}
	output.filter={
		Type=result["type"]
	}
	output.sort={
		Timestamp=message.mtime,
		Channel=message.mchannel,
		Count=message.count,
		Player=message.msender,
		Type=output.display.Type,
		Profession=output.display.Profession,
		Class=output.display.Class,
		Role=output.display.Role,
		Raid=output.display.Raid,
		Message=message.mmsg
	}
	LFMonitor.processedHistory[id] = output
	if LFMonitor.HistoryFrame then
		resort()
	end
end

local function addCells(row, columnList, totalWidth)
	local column, prevCell
	LFMonitor.historyTable[row] = {}
	for _, column in ipairs(columnList) do
		local cell = AceGUI:Create("InteractiveLabel")
		cell:SetFont("Fonts\\ARIALN.TTF", 12)
		if column.width then
			cell:SetWidth(column.width)
			totalWidth = totalWidth - column.width
		else
			cell:SetWidth(totalWidth)
			totalWidth = 0
		end
		cell:SetCallback("OnClick", function(self) ChatFrame1EditBox:SetText(LFMonitor.WHISPER_COMMAND .. " " .. self:GetUserData("player") .. " ") end)
		LFMonitor.historyRows[row]:AddChild(cell)
		cell.label:SetJustifyH(column.justify)
		LFMonitor.historyTable[row][column.name] = cell
		prevCell = cell
	end
end

function LFMonitor:showHistoryTable()
	local columns = {
		{name="Count",		width=10,	justify="RIGHT",	sort=-1},
		{name="Channel",	width=10,	justify="RIGHT",	sort=1},
		{name="Timestamp",	width=50,	justify="RIGHT",	sort=-1},
		{name="Player",		width=80,	justify="LEFT",		sort=1},
		{name="Type",		width=50,	justify="CENTER",	sort=1},
		{name="Raid",		width=80,	justify="CENTER",	sort=1},
		{name="Class",		width=50,	justify="CENTER",	sort=1},
		{name="Role",		width=50,	justify="CENTER",	sort=1},
		{name="Profession",	width=40,	justify="CENTER",	sort=1},
		{name="Message",				justify="LEFT",		sort=1},
	}
	local row, tex
	local window = AceGUI:Create("Window")
	LFMonitor.HistoryFrame = window
	window:SetWidth(910)
	window:SetHeight(640)
	window:SetLayout("List")
	local boxGroup = AceGUI:Create("InlineGroup")
	window:AddChild(boxGroup)
	boxGroup:SetTitle(LFMonitor.SHOW_TYPES)
	boxGroup:SetFullWidth(true)
	boxGroup:SetLayout("Flow")
	local k,v,box
	for k,v in pairs(LFMonitor.ABBRS["type"]) do
		box = AceGUI:Create("CheckBox")
		boxGroup:AddChild(box)
		box:SetLabel(v)
		box:SetWidth(60)
		box:SetValue(LFMonitor.historyFilter[k])
		box:SetCallback("OnValueChanged", function(box)
			LFMonitor.historyFilter[k] = box:GetValue()
			resort()
		end)
	end
	local headerRow = AceGUI:Create("SimpleGroup")
	headerRow:SetFullWidth(true)
	headerRow:SetHeight(16)
	headerRow:SetLayout("Flow")
	window:AddChild(headerRow)
	local headerTexture = headerRow.frame:CreateTexture(nil, "ARTWORK")
	headerTexture:SetPoint("TOPLEFT", headerRow.frame, "TOPLEFT")
	headerTexture:SetPoint("BOTTOMRIGHT", headerRow.frame, "BOTTOMRIGHT")
	headerTexture:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	headerTexture:SetTexCoord(0.1, 0.8, 0, 1)
	headerTexture:SetVertexColor(1,1,1, 0.8)
	local scrollcontainer = AceGUI:Create("SimpleGroup") -- "InlineGroup" is also good
	scrollcontainer:SetFullWidth(true)
	scrollcontainer:SetFullHeight(true)
	scrollcontainer:SetLayout("Fill") -- important!
	local scroll = AceGUI:Create("ScrollFrame")
	scroll:SetFullWidth(true)
	scroll:SetFullHeight(true)
	scroll:SetLayout("List")
	window:SetTitle("LFMonitor Chat History")
	window:AddChild(scroll)

	local i
	for i = 1, LFMonitor.historyMax do
		row = AceGUI:Create("SimpleGroup")
		row:SetAutoAdjustHeight(false)
		row:SetFullWidth(true)
		LFMonitor.historyRows[i] = row
		scroll:AddChild(row)
		row:SetLayout("Flow")
		tex = row.frame:CreateTexture()
		tex:SetPoint("TOPLEFT", row.frame, "TOPLEFT")
		tex:SetPoint("BOTTOMRIGHT", row.frame, "BOTTOMRIGHT")
		row:SetUserData("texture", tex)
		addCells(i, columns, 860)
	end
	NUM_BROWSE_TO_DISPLAY = NEW_NUM_BROWSE

	local function selectHeader(header)
		local key = header:GetUserData("id")
		if LFMonitor.sortKey == key then
			LFMonitor.sortDir = -LFMonitor.sortDir
--			print(LFMonitor.sortDir)
			reverseSort()
		else
			local defaultDir = header:GetUserData("defaultDir")
--			print("defaultDir",defaultDir)
			LFMonitor.sortKey = key
			LFMonitor.sortDir = defaultDir
--			print(LFMonitor.sortDir)
			resort()
		end
	end

	local function createHeader(id, text, width, dir)
		local header = AceGUI:Create("InteractiveLabel")
		headerRow:AddChild(header)
		header:SetHeight(16)
		if width then
			header:SetWidth(width)
		end
		header:SetText(text)
		header:SetUserData("id", id)
		header:SetUserData("defaultDir", dir)
		header:SetCallback("OnClick", selectHeader)
	end

	local i, column, prevCell
	for i, column in ipairs(columns) do
		createHeader(column.name, LFMonitor.HISTORY_COLUMN_HEADERS[column.name], column.width, column.sort)
	end
	window:SetCallback("OnClose", function(widget)
		AceGUI:Release(widget)
		LFMonitor.HistoryFrame = nil
		end)
	resort()
end
-- print("Modules/HistoryFrame.lua end")
